CREATE TABLE Partner (
	PID INTEGER UNIQUE NOT NULL,
	MNAME VARCHAR(30) PRIMARY KEY
);

CREATE TABLE VPartner (
	MNAME VARCHAR(30) PRIMARY KEY REFERENCES Partner(MNAME),
	PREIS NUMERIC NOT NULL,
	ANTEIL NUMERIC(5,4) NOT NULL
);

CREATE TABLE BPartner (
	MNAME VARCHAR(30) PRIMARY KEY REFERENCES Partner(MNAME),
	GEBUEHR NUMERIC(4,3) NOT NULL
);

CREATE TABLE gibt_Rabatt (
	VMNAME VARCHAR(30) NOT NULL REFERENCES VPartner(MNAME),
	BMNAME VARCHAR(30) NOT NULL REFERENCES BPartner(MNAME),
	PROZENT NUMERIC(5,4) NOT NULL,
	PRIMARY KEY(VMNAME, BMNAME)
);

CREATE TABLE Account (
	MNAME VARCHAR(30) NOT NULL REFERENCES BPartner(MNAME),
	BENUTZER VARCHAR(20),
	PW VARCHAR(24),
	PRIMARY KEY(MNAME, BENUTZER)
);

CREATE TABLE Login (
	MNAME VARCHAR(30) NOT NULL,
	BENUTZER VARCHAR(20),
	DATUM TIMESTAMP,
	ID INTEGER,
	IP VARCHAR(15) NOT NULL,
	SUCCESS BOOLEAN NOT NULL,
	PRIMARY KEY(MNAME, BENUTZER, DATUM, ID),
	FOREIGN KEY(MNAME, BENUTZER) REFERENCES Account(MNAME, BENUTZER)
);

CREATE TABLE Kunde (
	VNAME VARCHAR(30) NOT NULL,
	NNAME VARCHAR(30) NOT NULL,
	GEBDAT DATE NOT NULL,
	KNR INTEGER PRIMARY KEY,
	IBAN VARCHAR(34) NOT NULL,
	UNIQUE(VNAME, NNAME, GEBDAT)
);

CREATE TABLE Karte (
	KNR INTEGER PRIMARY KEY,
	CVC INTEGER NOT NULL,
	ABLAUFDATUM DATE NOT NULL,
	MNAME VARCHAR(30) NOT NULL REFERENCES VPartner(MNAME),
	KUNDE INTEGER NOT NULL REFERENCES KUNDE(KNR)
);

CREATE TABLE ersetzt (
	VGKNR INTEGER PRIMARY KEY REFERENCES Karte(KNR),
	NFKNR INTEGER NOT NULL REFERENCES Karte(KNR),
	AB DATE NOT NULL
);

CREATE TABLE SecLevel (
	NAME VARCHAR(15),
	NR INTEGER,
	PRIMARY KEY(NAME, NR)
);

CREATE TABLE Transaktion (
	KNR INTEGER REFERENCES Karte(KNR),
	MNAME VARCHAR(30) NOT NULL,
	BENUTZER VARCHAR(20),
	TID INTEGER,
	TOTAL NUMERIC(10,2) NOT NULL,
	DATUM DATE NOT NULL,
	SLNAME VARCHAR(15) NOT NULL,
	SLNR INTEGER NOT NULL,
	PRIMARY KEY(KNR, MNAME, BENUTZER, TID),
	FOREIGN KEY(MNAME, BENUTZER) REFERENCES Account(MNAME, BENUTZER),
	FOREIGN KEY(SLNAME, SLNR) REFERENCES SecLevel(NAME, NR)
);

CREATE TABLE Mitarbeiter (
	SVNR INTEGER PRIMARY KEY,
	GEHALT NUMERIC(10,2) NOT NULL,
	VNAME VARCHAR(30) NOT NULL,
	NNAME VARCHAR(30) NOT NULL,
	GEBDAT DATE NOT NULL
);

CREATE TABLE hat_Zugriff (
	SVNR INTEGER REFERENCES Mitarbeiter(SVNR),
	MNAME VARCHAR(30),
	BENUTZER VARCHAR(20),
	PRIMARY KEY(SVNR, MNAME, BENUTZER),
	FOREIGN KEY(MNAME, BENUTZER) REFERENCES Account(MNAME, BENUTZER)
);

CREATE TABLE SecurityAgent(
	AID INTEGER PRIMARY KEY,
	TLEVEL NUMERIC(4,2) NOT NULL,
	NAME VARCHAR(50)
);

CREATE TABLE zugelassen (
	SVNR INTEGER REFERENCES Mitarbeiter(SVNR),
	SLNAME VARCHAR(15),
	SLNR INTEGER,
	AID INTEGER REFERENCES SecurityAgent(AID),
	PRIMARY KEY(SVNR, SLNAME, SLNR, AID),
	FOREIGN KEY(SLNAME, SLNR) REFERENCES SecLevel(NAME, NR)
);

CREATE TABLE Verdacht (
	KNR INTEGER,
	MNAME VARCHAR(30),
	BENUTZER VARCHAR(20),
	TID INTEGER,
	DATUM DATE NOT NULL,
	BEM VARCHAR(70) NOT NULL,
	TYP INTEGER NOT NULL,
	AID INTEGER NOT NULL REFERENCES SecurityAgent(AID),
	PRIMARY KEY(KNR, MNAME, BENUTZER, TID),
	FOREIGN KEY(KNR, MNAME, BENUTZER, TID) REFERENCES Transaktion(KNR, MNAME, BENUTZER, TID)
);

CREATE TABLE UserFeedback (
	KNR INTEGER,
	MNAME VARCHAR(30),
	BENUTZER VARCHAR(20),
	TID INTEGER,
	DATUM DATE NOT NULL,
	OK BOOLEAN NOT NULL,
	PRIMARY KEY(KNR, MNAME, BENUTZER, TID),
	FOREIGN KEY(KNR, MNAME, BENUTZER, TID) REFERENCES Verdacht(KNR, MNAME, BENUTZER, TID)
);